
/* ************************************************************************ *
 * samelink - Read the output from samefile output and links identical      *
 *            files together. The matches that coudn't be processed are     *
 *            printed on the output.                                        *
 *                                                                          *
 * Hard link identical files, keepin the last modified files:               *
 *            find / | samefile -iZt | samelink -Zt                         *
 *                                                                          *
 * Create symlinks to between files that are not on the same filesystem     *
 *            find / | samefile -i | samelink -s                            *
 * ************************************************************************ * 
 *              Written by Alex de Kruijff                2009              *
 * ************************************************************************ * 
 * This source was written with a tabstop every four characters             * 
 * In vi type :set ts=4                                                     * 
 * ************************************************************************ */

#include <err.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "mainAction.h"

static int digital = 0, value;
static unsigned long set, clear;

/**
 * Removes one of the files and replaces it with a hard- of symlink to the
 * other.
 *
 * @returns PRINT_AGAIN          - always need to continue
 */
int mychflags(const char *left, const char *right,
	const struct stat &s1, const struct stat &s2,
	const char *dst, const char *src, char *backup)
{
	// check if files are already hard linked
	if (s1.st_dev == s2.st_dev && s1.st_ino == s2.st_ino)
		return DISMISS_SILENTLY;

	// can wel go a head?
	if (!S_DRYRUN(flags))
	{
		if (digital)
		{
			lchflags(left, value);
			lchflags(right, value);
		}
		else
		{
			lchflags(left, (s1.st_flags | set) & clear);
			lchflags(right, (s2.st_flags | set) & clear);
		}
		return PRINT_AGAIN;
	}

	return src == left ? SUCCES_LEFT : SUCCES_RIGHT;
}

int main(int argc, char **argv)
{
	actionProcessOptions(argc, argv, "chflags");
	actionProcessInput(mychflags);
	if (S_VERBOSE_LEVEL3(flags))
		actionProcessStats();

	int argi = actionProcessOptions(argc, argv, "chflags");
	if (argc <= argi)
		actionUsage("chflags");

	// get arguments
	if (*argv[argi] >= '0' && *argv[argi])
	{
		char *end;
		errno = 0;
		value = strtol(argv[argi], &end, 8);
		if (value < 0)
			errno = ERANGE;
		if (errno)
			err(1, "invalid flags: %s", argv[argi]);
		if (*end)
			errx(1, "invalid flags: %s", argv[argi]);
		digital = 1;
	}
	else
	{
		if (strtofflags(&argv[argi], &set, &clear))
			errx(1, "invalid flag: %s", argv[argi]);
		clear = ~clear;
	}

	// work
	actionProcessInput(mychflags);
	if (S_VERBOSE_LEVEL3(flags))
		actionProcessStats();
}

